@@ -9,17 +9,24 @@ use Data::Dumper;
 $Data::Dumper::Sortkeys = 1;
 use Storable qw(dclone);
 use feature 'switch';
+use GooCanvas2;
 use Gscan2pdf::ComboBoxText;
 use Gscan2pdf::Dialog;
 use Gscan2pdf::Scanner::Options;
 use Gscan2pdf::Scanner::Profile;
 use Gscan2pdf::Translation '__';    # easier to extract strings with xgettext
 use List::MoreUtils qw(first_index);
+use Readonly;
+Readonly my $PAPER_TOLERANCE  => 1;
+Readonly my $OPTION_TOLERANCE => 0.001;
 
 my (
     $MAX_PAGES,        $MAX_INCREMENT, $DOUBLE_INCREMENT,
     $CANVAS_SIZE,      $CANVAS_BORDER, $CANVAS_POINT_SIZE,
-    $CANVAS_MIN_WIDTH, $NO_INDEX,      $EMPTY
+    $CANVAS_MIN_WIDTH, $NO_INDEX,      $EMPTY,
+
+    # start page
+    $start,
 );
 
 # need to register this with Glib before we can use it below
@@ -267,12 +274,23 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
         0,                                                        # default
         [qw/readable/]                                            # flags
     ),
+    Glib::ParamSpec->scalar(
+        'document',                                               # name
+        'Document',                                               # nick
+        'Gscan2pdf::Document for new scans',                      # blurb
+        [qw/readable writable/]                                   # flags
+    ),
+    Glib::ParamSpec->scalar(
+        'cursor',                                                 # name
+        'Cursor',                                                 # nick
+        'name of current cursor',                                 # blurb
+        [qw/readable writable/]                                   # flags
+    ),
   ];
 
-our $VERSION = '2.3.0';
+our $VERSION = '2.5.7';
 
 my ( $d_sane, $logger );
-my $tolerance             = 1;
 my $SANE_NAME_SCAN_TL_X   = SANE_NAME_SCAN_TL_X;
 my $SANE_NAME_SCAN_TL_Y   = SANE_NAME_SCAN_TL_Y;
 my $SANE_NAME_SCAN_BR_X   = SANE_NAME_SCAN_BR_X;
@@ -291,6 +309,7 @@ sub INIT_INSTANCE {
 
     # Notebook to collate options
     $self->{notebook} = Gtk3::Notebook->new;
+    $self->{notebook}->set_scrollable(TRUE);
     $vbox->pack_start( $self->{notebook}, TRUE, TRUE, 0 );
 
     # Notebook page 1
@@ -352,6 +371,9 @@ sub INIT_INSTANCE {
             else {
                 $spin_buttonn->set_value($value);
             }
+
+            # Check that there is room in the list for the number of pages
+            $self->update_num_pages;
         }
     );
     $self->signal_connect(
@@ -397,12 +419,21 @@ sub INIT_INSTANCE {
     $spin_buttons->signal_connect(
         'value-changed' => sub {
             $self->set( 'page-number-start', $spin_buttons->get_value );
+            $self->update_start_page;
         }
     );
     $self->signal_connect(
         'changed-page-number-start' => sub {
             my ( $widget, $value ) = @_;
             $spin_buttons->set_value($value);
+            my $slist = $self->get('document');
+            if ( not defined $slist ) { return }
+            $self->set(
+                'max-pages',
+                $slist->pages_possible(
+                    $value, $self->get('page-number-increment')
+                )
+            );
         }
     );
 
@@ -427,6 +458,14 @@ sub INIT_INSTANCE {
         'changed-page-number-increment' => sub {
             my ( $widget, $value ) = @_;
             $spin_buttoni->set_value($value);
+            my $slist = $self->get('document');
+            if ( not defined $slist ) { return }
+            $self->set(
+                'max-pages',
+                $slist->pages_possible(
+                    $self->get('page-number-start'), $value
+                )
+            );
         }
     );
 
@@ -525,8 +564,8 @@ sub INIT_INSTANCE {
     my $vboxsp = Gtk3::VBox->new;
     $vboxsp->set_border_width($border_width);
     $framesp->add($vboxsp);
-    $self->{combobsp} = Gscan2pdf::ComboBoxText->new;
-    $self->{combobsp}->signal_connect(
+    $self->{combobsp}                = Gscan2pdf::ComboBoxText->new;
+    $self->{combobsp_changed_signal} = $self->{combobsp}->signal_connect(
         changed => sub {
             $self->{num_reloads} = 0;    # num-reloads is read-only
             $self->set( 'profile', $self->{combobsp}->get_active_text );
@@ -541,6 +580,11 @@ sub INIT_INSTANCE {
     $vbutton->signal_connect( clicked => \&_save_profile_callback, $self );
     $hboxsp->pack_start( $vbutton, TRUE, TRUE, 0 );
 
+    # Edit button
+    my $ebutton = Gtk3::Button->new_from_stock('gtk-edit');
+    $ebutton->signal_connect( clicked => \&_edit_profile_callback, $self );
+    $hboxsp->pack_start( $ebutton, FALSE, FALSE, 0 );
+
     # Delete button
     my $dbutton = Gtk3::Button->new_from_stock('gtk-delete');
     $dbutton->signal_connect(
@@ -550,7 +594,7 @@ sub INIT_INSTANCE {
     );
     $hboxsp->pack_start( $dbutton, FALSE, FALSE, 0 );
 
-    $self->add_actions(
+    ( $self->{scan_button} ) = $self->add_actions(
         __('Scan'),
         sub {
             $self->signal_emit('clicked-scan-button');
@@ -566,9 +610,40 @@ sub INIT_INSTANCE {
     $self->{setting_profile}              = [];
     $self->{setting_current_scan_options} = [];
 
+    # If we get an error opening a device, add it to a session-only blacklist,
+    # removing it from the device list
+    $self->{device_blacklist} = [];
+    $self->signal_connect( 'process-error' => \&process_error_callback );
+
+    $self->set( 'cursor', 'default' );
     return $self;
 }
 
+sub process_error_callback {
+    my ( $self, $process, $msg ) = @_;
+    if ( $process =~ /^(?:get_devices|open_device|find_scan_options)$/xsm ) {
+        my $device = $self->get('device');
+        if ( defined $device and $device ne $EMPTY ) {
+            $logger->warn("adding device '$device' to session blacklist");
+            push @{ $self->{device_blacklist} }, $device;
+            my $device_list = $self->get('device-list');
+            my @device_list;
+            for my $dev ( @{$device_list} ) {
+                my $found = FALSE;
+                for ( @{ $self->{device_blacklist} } ) {
+                    if ( $_ eq $dev->{name} ) {
+                        $found = TRUE;
+                        last;
+                    }
+                }
+                if ( not $found ) { push @device_list, $dev }
+            }
+            $self->set( 'device-list', \@device_list );
+        }
+    }
+    return;
+}
+
 sub _add_device_combobox {
     my ( $self, $vbox ) = @_;
     $self->{hboxd} = Gtk3::HBox->new;
@@ -673,6 +748,129 @@ sub _save_profile_callback {
     return;
 }
 
+# Clone current profile, display list of options, allowing user to delete those
+# not required, and then to cancel or accept the changes.
+
+sub _edit_profile_callback {
+    my ( $widget, $parent ) = @_;
+    my $name = $parent->get('profile');
+    my ( $msg, $profile );
+
+    if ( not defined $name or $name eq $EMPTY ) {
+        $msg     = __('Editing current scan options');
+        $profile = $parent->{current_scan_options};
+    }
+    else {
+        $msg = sprintf __('Editing scan profile "%s"'), $name;
+        $profile = $parent->{profiles}{$name};
+    }
+    my $dialog = Gtk3::Dialog->new(
+        $msg, $parent,
+        'destroy-with-parent',
+        'gtk-ok'     => 'ok',
+        'gtk-cancel' => 'cancel'
+    );
+    my $label = Gtk3::Label->new($msg);
+    $dialog->get_content_area->pack_start( $label, TRUE, TRUE, 0 );
+
+    # Clone so that we can cancel the changes, if necessary
+    $profile = dclone($profile);
+    _build_profile_table( $profile, $parent->get('available-scan-options'),
+        $dialog->get_content_area );
+    $dialog->set_default_response('ok');
+    $dialog->show_all;
+
+    # save the profile and reload
+    if ( $dialog->run eq 'ok' ) {
+        if ( not defined $name or $name eq $EMPTY ) {
+            $parent->set_current_scan_options($profile);
+        }
+        else {
+            $parent->{profiles}{$name} = $profile;
+
+            # unset profile to allow us to set it again on reload
+            $parent->{profile} = undef;
+
+            # emit signal to update settings
+            $parent->signal_emit( 'added-profile', $name,
+                $parent->{profiles}{$name} );
+            my $signal;
+            $signal = $parent->signal_connect(
+                'reloaded-scan-options' => sub {
+                    $parent->signal_handler_disconnect($signal);
+                    $parent->set_profile($name);
+                }
+            );
+            $parent->scan_options( $parent->get('device') );
+        }
+    }
+    $dialog->destroy;
+    return;
+}
+
+sub _build_profile_table {
+    my ( $profile, $options, $vbox ) = @_;
+
+    my $frameb = Gtk3::Frame->new( __('Backend options') );
+    my $framef = Gtk3::Frame->new( __('Frontend options') );
+    $vbox->pack_start( $frameb, TRUE, TRUE, 0 );
+    $vbox->pack_start( $framef, TRUE, TRUE, 0 );
+
+    # listbox to align widgets
+    my $listbox = Gtk3::ListBox->new;
+    $listbox->set_selection_mode('none');
+    $frameb->add($listbox);
+    my $iter = $profile->each_backend_option;
+    while ( my $i = $iter->() ) {
+        my ( $name, $val ) = $profile->get_backend_option_by_index($i);
+        my $opt   = $options->by_name($name);
+        my $row   = Gtk3::ListBoxRow->new;
+        my $hbox  = Gtk3::HBox->new;
+        my $label = Gtk3::Label->new( $d_sane->get( $opt->{title} ) );
+        $hbox->pack_start( $label, FALSE, TRUE, 0 );
+        my $button = Gtk3::Button->new_from_stock('gtk-delete');
+        $hbox->pack_end( $button, FALSE, FALSE, 0 );
+        $button->signal_connect(
+            clicked => sub {
+                $logger->debug("removing option '$name' from profile");
+                $profile->remove_backend_option_by_index($i);
+                $frameb->destroy;
+                $framef->destroy;
+                _build_profile_table( $profile, $options, $vbox );
+            }
+        );
+        $row->add($hbox);
+        $listbox->add($row);
+    }
+
+    $listbox = Gtk3::ListBox->new;
+    $listbox->set_selection_mode('none');
+    $framef->add($listbox);
+    $iter = $profile->each_frontend_option;
+    while ( my $name = $iter->() ) {
+        my $row   = Gtk3::ListBoxRow->new;
+        my $hbox  = Gtk3::HBox->new;
+        my $label = Gtk3::Label->new($name);
+        $hbox->pack_start( $label, FALSE, TRUE, 0 );
+        my $button = Gtk3::Button->new_from_stock('gtk-delete');
+        $hbox->pack_end( $button, FALSE, FALSE, 0 );
+        $button->signal_connect(
+            clicked => sub {
+                $logger->debug("removing option '$name' from profile");
+                $profile->remove_frontend_option($name);
+                $frameb->destroy;
+                $framef->destroy;
+                _build_profile_table( $profile, $options, $vbox );
+            }
+        );
+        $row->add($hbox);
+        $listbox->add($row);
+    }
+
+    $vbox->show_all;
+    return;
+}
+
 sub SET_PROPERTY {
     my ( $self, $pspec, $newval ) = @_;
     my $name   = $pspec->get_name;
@@ -685,9 +883,7 @@ sub SET_PROPERTY {
         $logger->debug('Set logger in Gscan2pdf::Dialog::Scan');
         $self->{$name} = $newval;
     }
-    elsif (( defined $newval and defined $oldval and $newval ne $oldval )
-        or ( defined $newval xor defined $oldval ) )
-    {
+    elsif ( _new_val( $oldval, $newval ) ) {
         my $msg;
         if ( defined $logger ) {
             $msg =
@@ -704,6 +900,10 @@ sub SET_PROPERTY {
             when ('available_scan_options') {
                 $self->_set_available_scan_options( $name, $newval );
             }
+            when ('cursor') {
+                $self->{$name} = $newval;
+                $self->set_cursor($newval);
+            }
             when ('device') {
                 $self->{$name} = $newval;
                 $self->set_device($newval);
@@ -714,6 +914,16 @@ sub SET_PROPERTY {
                 $self->set_device_list($newval);
                 $self->signal_emit( 'changed-device-list', $newval )
             }
+            when ('document') {
+                $self->{$name} = $newval;
+
+                # Update the start spinbutton if the page number is been edited.
+                my $slist = $self->get('document');
+                if ( defined $slist ) {
+                    $slist->get_model->signal_connect(
+                        'row-changed' => sub { $self->update_start_page } );
+                }
+            }
             when ('num_pages') {
                 $self->_set_num_pages( $name, $newval );
             }
@@ -768,8 +978,6 @@ sub SET_PROPERTY {
                     }
                 );
                 $self->set_paper($newval);
-                $self->{current_scan_options}
-                  ->add_frontend_option( $name, $newval );
             }
             when ('paper_formats') {
                 $self->{$name} = $newval;
@@ -805,6 +1013,14 @@ sub SET_PROPERTY {
     return;
 }
 
+sub _new_val {
+    my ( $oldval, $newval ) = @_;
+    return (
+             ( defined $newval and defined $oldval and $newval ne $oldval )
+          or ( defined $newval xor defined $oldval )
+    );
+}
+
 sub _flatbed_or_duplex_callback {
     my ($self) = @_;
     my $options = $self->get('available-scan-options');
@@ -918,6 +1134,7 @@ sub show {
     {
         $self->hide_geometry( $self->get('available-scan-options') );
     }
+    $self->set_cursor;
     return;
 }
 
@@ -1007,22 +1224,22 @@ sub pack_widget {
             my $text;
             given ( $opt->{unit} ) {
                 when (SANE_UNIT_PIXEL) {
-                    $text = $d_sane->get('pel')
+                    $text = __('pel')
                 }
                 when (SANE_UNIT_BIT) {
-                    $text = $d_sane->get('bit')
+                    $text = __('bit')
                 }
                 when (SANE_UNIT_MM) {
-                    $text = $d_sane->get('mm')
+                    $text = __('mm')
                 }
                 when (SANE_UNIT_DPI) {
-                    $text = $d_sane->get('dpi')
+                    $text = __('ppi')
                 }
                 when (SANE_UNIT_PERCENT) {
-                    $text = $d_sane->get(q{%})
+                    $text = __(q{%})
                 }
                 when (SANE_UNIT_MICROSECOND) {
-                    $text = $d_sane->get('us')
+                    $text = __('μs')
                 }
             }
             my $label = Gtk3::Label->new($text);
@@ -1122,7 +1339,8 @@ sub create_paper_widget {
             }
         );
 
-        # If the geometry is changed, unset the paper size
+        # If the geometry is changed, unset the paper size,
+        # if we are not setting a profile
         for (
             ( SANE_NAME_SCAN_TL_X, SANE_NAME_SCAN_TL_Y,
                 SANE_NAME_SCAN_BR_X,   SANE_NAME_SCAN_BR_Y,
@@ -1134,7 +1352,9 @@ sub create_paper_widget {
                 my $widget = $self->{option_widgets}{$_};
                 $widget->signal_connect(
                     changed => sub {
-                        if ( defined $self->get('paper') ) {
+                        if ( not @{ $self->{setting_current_scan_options} }
+                            and defined $self->get('paper') )
+                        {
                             $self->set( 'paper', undef );
                         }
                     }
@@ -1250,7 +1470,7 @@ sub update_options {
     }
 
     # Reapply current options to ensure the same values are still set.
-    $self->set_current_scan_options($current_scan_options);
+    $self->add_current_scan_options($current_scan_options);
 
     # In case the geometry values have changed,
     # update the available paper formats
@@ -1367,7 +1587,8 @@ sub set_paper_formats {
         $self->{ignored_paper_formats} = ();
         my $options = $self->get('available-scan-options');
         for ( keys %{$formats} ) {
-            if ( $options->supports_paper( $formats->{$_}, $tolerance ) ) {
+            if ( $options->supports_paper( $formats->{$_}, $PAPER_TOLERANCE ) )
+            {
                 $logger->debug("Options support paper size '$_'.");
                 $combobp->prepend_text($_);
             }
@@ -1441,6 +1662,7 @@ sub set_paper {
     if ( not $paper_profile->num_backend_options ) {
         $self->hide_geometry($options);
         $self->{paper} = $paper;
+        $self->{current_scan_options}->add_frontend_option( 'paper', $paper );
         $self->signal_emit( 'changed-paper', $paper );
         return;
     }
@@ -1448,17 +1670,21 @@ sub set_paper {
     my $signal;
     $signal = $self->signal_connect(
         'changed-current-scan-options' => sub {
-            $self->signal_handler_disconnect($signal);
-            $self->hide_geometry($options);
-            $self->{paper} = $paper;
-            $self->set( 'profile', undef );
-            $self->signal_emit( 'changed-paper', $paper );
+            my ( $dialog, $profile, $uuid ) = @_;
+            if ( $paper_profile->{uuid} eq $uuid ) {
+                $self->signal_handler_disconnect($signal);
+                $self->hide_geometry($options);
+                $self->{paper} = $paper;
+                $self->{current_scan_options}
+                  ->add_frontend_option( 'paper', $paper );
+                $self->signal_emit( 'changed-paper', $paper );
+            }
         }
     );
 
     # Don't trigger the changed-paper signal
     # until we have finished setting the profile
-    $self->set_current_scan_options($paper_profile);
+    $self->add_current_scan_options($paper_profile);
     return;
 }
 
@@ -1644,7 +1870,13 @@ sub edit_paper {
 sub save_current_profile {
     my ( $self, $name ) = @_;
     $self->add_profile( $name, $self->{current_scan_options} );
+
+    # Block signal or else we fire another round of profile loads
+    $self->{combobsp}->signal_handler_block( $self->{combobsp_changed_signal} );
     $self->{combobsp}->set_active( $self->{combobsp}->get_num_rows - 1 );
+    $self->{combobsp}
+      ->signal_handler_unblock( $self->{combobsp_changed_signal} );
+    $self->{profile} = $name;
     return;
 }
 
@@ -1803,9 +2035,8 @@ sub set_options {
     my $window = Gscan2pdf::Dialog->new(
         'transient-for' => $self,
         title           => $d_sane->get( $opt->{title} ),
-        destroy         => TRUE,
     );
-    my $canvas = Goo::Canvas->new;
+    my $canvas = GooCanvas2::Canvas->new;
     $canvas->set_size_request( $CANVAS_SIZE, $CANVAS_SIZE );
     $canvas->{border} = $CANVAS_BORDER;
     $window->get_content_area->add($canvas);
@@ -1824,8 +2055,15 @@ sub set_options {
             my $fleur = Gtk3::Gdk::Cursor->new('fleur');
             my ( $x, $y ) = to_graph( $widget, $event->x, $event->y );
             $x = int($x) + 1;
-            splice @{ $widget->{val} }, $x, 0, $y;
-            splice @{ $widget->{items} }, $x, 0, add_value( $root, $widget );
+            if ( $x > @{ $widget->{val} } ) {
+                push @{ $widget->{val} }, $y;
+                push @{ $widget->{items} }, add_value( $root, $widget );
+            }
+            else {
+                splice @{ $widget->{val} }, $x, 0, $y;
+                splice @{ $widget->{items} }, $x, 0,
+                  add_value( $root, $widget );
+            }
             update_graph($widget);
             return TRUE;
         }
@@ -1851,7 +2089,6 @@ sub set_options {
             return FALSE;
         }
     );
-    $canvas->can_focus(TRUE);
     $canvas->grab_focus($root);
 
     $canvas->{opt} = $opt;
@@ -1866,25 +2103,18 @@ sub set_options {
           sort { $a <=> $b } @{ $opt->{constraint} };
     }
 
-    # Apply button
-    my $abutton = Gtk3::Button->new_from_stock('gtk-apply');
-    $window->add_button($abutton);
-    $abutton->signal_connect(
-        clicked => sub {
+    $window->add_actions(
+        'gtk-apply' => sub {
             $self->set_option( $opt, $canvas->{val} );
 
  # when INFO_INEXACT is implemented, so that the value is reloaded, check for it
  # here, so that the reloaded value is not overwritten.
             $opt->{val} = $canvas->{val};
             $window->destroy;
-        }
+        },
+        'gtk-cancel' => sub { $window->destroy }
     );
 
-    # Cancel button
-    my $cbutton = Gtk3::Button->new_from_stock('gtk-cancel');
-    $window->add_button($cbutton);
-    $cbutton->signal_connect( clicked => sub { $window->destroy } );
-
 # Have to show the window before updating it otherwise is doesn't know how big it is
     $window->show_all;
     update_graph($canvas);
@@ -1893,8 +2123,12 @@ sub set_options {
 
 sub add_value {
     my ( $root, $canvas ) = @_;
-    my $item = Goo::Canvas::Rect->new(
-        $root, 0, 0, $CANVAS_POINT_SIZE, $CANVAS_POINT_SIZE,
+    my $item = GooCanvas2::CanvasRect->new(
+        parent       => $root,
+        x            => 0,
+        y            => 0,
+        width        => $CANVAS_POINT_SIZE,
+        height       => $CANVAS_POINT_SIZE,
         'fill-color' => 'black',
         'line-width' => 0,
     );
@@ -2034,7 +2268,7 @@ sub update_graph {
       ( $xbounds[1] - $xbounds[0], $ybounds[1] - $ybounds[0] );
 
     # Calculate bounds of canvas
-    my ( $x, $y, $cwidth, $cheight ) = $canvas->allocation->values;
+    my ( $x, $y, $cwidth, $cheight ) = $canvas->get_bounds;
 
     # Calculate scale factors
     my @scale = (
@@ -2074,16 +2308,54 @@ sub set_current_scan_options {
         return;
     }
 
+    # If we have no options set, no need to reset to defaults
+    if ( $self->{current_scan_options}->num_backend_options == 0 ) {
+        $self->add_current_scan_options($profile);
+        return;
+    }
+
+    # reload to get defaults before applying profile
+    my $signal;
+    $self->{current_scan_options} =
+      Gscan2pdf::Scanner::Profile->new_from_data( dclone($profile) );
+    $signal = $self->signal_connect(
+        'reloaded-scan-options' => sub {
+            $self->signal_handler_disconnect($signal);
+            $self->add_current_scan_options($profile);
+        }
+    );
+    $self->scan_options( $self->get('device') );
+    return;
+}
+
+# Apply options referenced by hashref without resetting existing options
+
+sub add_current_scan_options {
+    my ( $self, $profile ) = @_;
+    if ( not defined $profile ) {
+        $logger->error('Cannot add undefined profile');
+        return;
+    }
+    elsif ( ref($profile) ne 'Gscan2pdf::Scanner::Profile' ) {
+        $logger->error(
+            ref($profile) . ' is not a Gscan2pdf::Scanner::Profile object' );
+        return;
+    }
+
     # First clone the profile, as otherwise it would be self-modifying
     my $clone = dclone($profile);
 
+    # forget the previous option info calls, as these are only interesting
+    # *whilst* setting a profile, and now we are starting from scratch
+    delete $self->{option_info};
+
     push @{ $self->{setting_current_scan_options} }, $clone->{uuid};
 
     # Give the GUI a chance to catch up between settings,
     # in case they have to be reloaded.
     # Use the callback to trigger the next loop
     $self->_set_option_profile( $clone, $clone->each_backend_option );
-    return $clone->{uuid};
+    return;
 }
 
 sub _set_option_profile {
@@ -2111,8 +2383,24 @@ sub _set_option_profile {
             }
         }
 
+   # Ignore option if info from previous set_option() reported SANE_INFO_INEXACT
+        if ( defined $self->{option_info}{ $opt->{name} }
+            and $self->{option_info}{ $opt->{name} } & SANE_INFO_INEXACT )
+        {
+            $logger->warn(
+"Skip setting option '$name' to '$val', as previous call set SANE_INFO_INEXACT"
+            );
+            $self->_set_option_profile( $profile, $next );
+            return;
+        }
+
         # Ignore option if value already within tolerance
-        if ( Gscan2pdf::Scanner::Options::within_tolerance( $opt, $val ) ) {
+        if (
+            Gscan2pdf::Scanner::Options::within_tolerance(
+                $opt, $val, $OPTION_TOLERANCE
+            )
+          )
+        {
             $logger->info(
                 "No need to set option '$name': already within tolerance.");
             $self->_set_option_profile( $profile, $next );
@@ -2146,6 +2434,10 @@ sub _set_option_profile {
     else {
 
         # Having set all backend options, set the frontend options
+        # Set paper formats first to make sure that any paper required is
+        # available
+        $self->set_paper_formats( $self->{paper_formats} );
+
         my $iter = $profile->each_frontend_option;
         while ( my $key = $iter->() ) {
             $self->set( $key, $profile->get_frontend_option($key) );
@@ -2259,6 +2551,96 @@ sub get_xy_resolution {
     return $x, $y;
 }
 
+# Update the number of pages to scan spinbutton if necessary
+
+sub update_num_pages {
+    my ($self) = @_;
+    my $slist = $self->get('document');
+    if ( not defined $slist ) { return }
+    my $n = $slist->pages_possible( $self->get('page-number-start'),
+        $self->get('page-number-increment') );
+    if ( $n > 0 and $n < $self->get('num-pages') ) {
+        $self->set( 'num-pages', $n );
+    }
+    return;
+}
+
+# Called either from changed-value signal of spinbutton,
+# or row-changed signal of simplelist
+
+sub update_start_page {
+    my ($self) = @_;
+    my $slist = $self->get('document');
+    if ( not defined $slist ) { return }
+    my $value = $self->get('page-number-start');
+    if ( not defined $start ) { $start = $self->get('page-number-start') }
+    my $step = $value - $start;
+    if ( $step == 0 ) { $step = $self->get('page-number-increment') }
+    $start = $value;
+
+    while ( $slist->pages_possible( $value, $step ) == 0 ) {
+        if ( $value < 1 ) {
+            $value = 1;
+            $step  = 1;
+        }
+        else {
+            $value += $step;
+        }
+    }
+    $self->set( 'page-number-start', $value );
+    $start = $value;
+
+    $self->update_num_pages;
+    return;
+}
+
+# Reset start page number after delete or new
+
+sub reset_start_page {
+    my ($self) = @_;
+    my $slist = $self->get('document');
+    if ( not defined $slist ) { return }
+    if ( $#{ $slist->{data} } > $NO_INDEX ) {
+        my $start_page = $self->get('page-number-start');
+        my $step       = $self->get('page-number-increment');
+        if ( $start_page > $slist->{data}[ $#{ $slist->{data} } ][0] + $step ) {
+            $self->set( 'page-number-start',
+                $slist->{data}[ $#{ $slist->{data} } ][0] + $step );
+        }
+    }
+    else {
+        $self->set( 'page-number-start', 1 );
+    }
+    return;
+}
+
+sub set_cursor {
+    my ( $self, $cursor ) = @_;
+    my $win = $self->get_window;
+    if ( not defined $cursor ) {
+        $cursor = $self->get('cursor');
+    }
+    if ( defined $win ) {
+        my $display = Gtk3::Gdk::Display::get_default;
+        $win->set_cursor(
+            Gtk3::Gdk::Cursor->new_from_name( $display, $cursor ) );
+    }
+    $self->{scan_button}->set_sensitive( $cursor eq 'default' );
+    return;
+}
+
+sub get_label_for_option {
+    my ( $self, $name ) = @_;
+    my $widget = $self->{option_widgets}{$name};
+    my $hbox   = $widget->get_parent;
+    for my $child ( $hbox->get_children ) {
+        if ( $child->isa('Gtk3::Label') ) {
+            return $child->get_text;
+        }
+    }
+    return;
+}
+
 1;
 
 __END__
