@@ -41,7 +41,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::Scan::, properties => [
     ),
 ];
 
-our $VERSION = '2.5.5';
+our $VERSION = '2.5.7';
 
 my $SANE_NAME_SCAN_TL_X   = SANE_NAME_SCAN_TL_X;
 my $SANE_NAME_SCAN_TL_Y   = SANE_NAME_SCAN_TL_Y;
@@ -79,6 +79,7 @@ sub SET_PROPERTY {
 
 sub get_devices {
     my ($self) = @_;
+    $self->set( 'cursor', 'wait' );
 
     my $pbar;
     my $hboxd = $self->{hboxd};
@@ -113,15 +114,17 @@ sub get_devices {
             }
             $self->set( 'device-list', \@device_list );
             $hboxd->show_all;
+            $self->set( 'cursor', 'default' );
         }
     );
     return;
 }
 
-# Scan device-dependent scan options
+# retrieve device-dependent scan options
 
 sub scan_options {
     my ($self) = @_;
+    $self->set( 'cursor', 'wait' );
 
     # Remove any existing pages
     while ( $self->{notebook}->get_n_pages > 1 ) {
@@ -168,12 +171,14 @@ sub scan_options {
                     # so don't set this until we have finished
                     $self->set( 'available-scan-options', $options );
                     $self->set_paper_formats( $self->{paper_formats} );
+                    $self->set( 'cursor', 'default' );
                 },
                 sub {    # error callback
                     my ($message) = @_;
                     $self->signal_emit( 'process-error',
                         'find_scan_options',
                         __( 'Error retrieving scanner options: ' . $message ) );
+                    $self->set( 'cursor', 'default' );
                 }
             );
         },
@@ -181,6 +186,7 @@ sub scan_options {
             my ($message) = @_;
             $self->signal_emit( 'process-error', 'open_device',
                 __( 'Error opening device: ' . $message ) );
+            $self->set( 'cursor', 'default' );
         }
     );
     return;
@@ -250,7 +256,11 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
 
             # Label
             if ( $opt->{type} != SANE_TYPE_BUTTON ) {
-                my $label = Gtk3::Label->new( $d_sane->get( $opt->{title} ) );
+                my $text = $opt->{title};
+                if ( not defined $text or $text eq $EMPTY ) {
+                    $text = $opt->{name};
+                }
+                my $label = Gtk3::Label->new( $d_sane->get($text) );
                 $hbox->pack_start( $label, FALSE, FALSE, 0 );
             }
 
@@ -478,6 +488,7 @@ sub set_option {
 
 sub scan {
     my ($self) = @_;
+    $self->set( 'cursor', 'progress' );
 
     # Get selected number of pages
     my $npages = $self->get('num-pages');
@@ -517,6 +528,7 @@ sub scan {
         },
         finished_callback => sub {
             $self->signal_emit( 'finished-process', 'scan_pages' );
+            $self->set( 'cursor', 'default' );
 
             if ( $self->get('cycle-sane-handle') ) {
                 my $current = $self->get('current-scan-options');
@@ -541,6 +553,7 @@ sub scan {
         error_callback => sub {
             my ($msg) = @_;
             $self->signal_emit( 'process-error', 'scan_pages', $msg );
+            $self->set( 'cursor', 'default' );
         }
     );
     return;
